<?php

namespace Drupal\syncart\Hook;

use Drupal\taxonomy\Entity\Term;
use Drupal\commerce_price\Price;

/**
 * Hook preprocess page class.
 */
class PreprocessCommerceProduct {

  /**
   * Hook.
   */
  public static function hook(&$variables) {
    $variables['cart'] = [
      '#theme' => 'syncart-cart-field',
      '#data' => self::getProductVariation($variables['product_entity']),
    ];

    $variables['favorite'] = [
      '#theme' => 'syncart-favorite-field',
      '#data' => [
        'uid' => \Drupal::currentUser()->id(),
        'product' => $variables['product_entity'],
      ],
    ];
  }

  /**
   * Get product variation price.
   */
  public static function getProductVariation($product) {
    $unit_name = '';
    $qfloat = 0;
    $qstep = 1;
    if ($product->hasField('field_unit')) {
      $unit = $product->get('field_unit')->getValue();
      if (!empty($unit)) {
        $term = Term::load($unit[0]['target_id']);
        $unit_name = $term->getName();
        $qfloat = (int) $term->get('field_unit_float')->getValue()[0]['value'];
        if ($qfloat != 0) {
          $config = \Drupal::config('core.entity_form_display.commerce_order_item.default.default');
          $qstep = round($config->getRawData()['content']['quantity']['settings']['step'], 2);
        }
      }
    }
    $result = FALSE;
    $full = 1;
    $config = \Drupal::config('syncart.settings');
    $check_store = $config->get('check_store');
    if (is_null($check_store)) {
      $check_store = 0;
    }
    $novariation = $config->get('no_variation_text');
    if (is_null($novariation)) {
      $novariation = '';
    }
    $showquantity = $config->get('show_quantity');
    if (is_null($showquantity)) {
      $showquantity = 0;
    }
    $currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    $currency_formatter = \Drupal::service('commerce_price.currency_formatter');
    $prices = [];
    $attributes = [];
    foreach ($product->variations as $item) {
      $variation = $item->entity;
      if (!is_object($variation->field_stock)) {
        $stock = 0;
      }
      else {
        $stock = (int) $variation->field_stock->value;
      }
      if (!is_object($variation)) {
        continue;
      }
      $priceObject = $variation->getPrice();
      if (!is_object($priceObject)) {
        continue;
      }
      $attribute_item = [];
      foreach ($variation->getAttributeValues() as $key => $value) {
        $attribute_item[] = [
          'id' => $value->id(),
          'label' => $value->getAttribute()->label(),
          'name' => $value->getName(),
        ];
        if ($check_store != 0 && $stock == 0) {
          continue;
        }
        $attribute = [
          'id' => $value->id(),
          'name' => $value->getName(),
        ];
        if (!array_key_exists($value->getAttribute()->label(), $attributes)) {
          $attributes[$value->getAttribute()->label()][] = $attribute;
        }
        if (!in_array($attribute, $attributes[$value->getAttribute()->label()])) {
          $attributes[$value->getAttribute()->label()][] = $attribute;
        }
      }
      $price = $priceObject->getNumber();
      $currency_code = $variation->getPrice()->getCurrencyCode();
      /** @var \Drupal\commerce_price\Entity\CurrencyInterface $currency */
      $currency = $currency_storage->load($currency_code);
      $old = FALSE;
      if ($variation->field_oldprice && $variation->field_oldprice->value) {
        $old_price = $variation->field_oldprice->value;
        $old = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => new Price($old_price, $currency_code),
          ],
        ];
      }

      $prices[] = [
        'variation_id' => $variation->id(),
        'price' => $variation->getPrice(),
        'old_price' => $old,
        'currency_code' => $currency_code,
        'price_format' => !empty($price) ? $currency_formatter->format($price, $currency_code, []) : '',
        'oldprice_format' => !empty($old_price) ? $currency_formatter->format($old_price, $currency_code, []) : '',
        'currency' => $currency->getSymbol(),
        'attributes' => $attribute_item,
        'stock' => $stock,
      ];
      $routeName = \Drupal::routeMatch()->getRouteName();
      if ($routeName != 'entity.commerce_product.canonical') {
        $full = 0;
        break;
      }
    }
    $result = [
      'product' => $product,
      'settings' => [
        'unit' => $unit_name,
        'qstep' => $qstep,
        'fullstate' => $full,
        'checkstore' => $check_store,
        'showquantity' => $showquantity,
        'novariation' => $novariation,
      ],
      'prices' => $prices,
      'attributes' => $attributes,
      'uid' => \Drupal::currentUser()->id(),
    ];
    return $result;
  }

}
